<?php
namespace Teknisa\FIN\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;



class TalaoCheque extends \Teknisa\FIN\Controllers\TeknisaCrud {
    protected $entityManager;
    protected $dataSourceName = "talaocheque_operador";
    protected $environment;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        parent::__construct($crudService, $parameterBag);
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }

    public function delete(DTO\Request $request, DTO\Response $response) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();

        $r = $request->getRow();
        $r->set('NRORG',$this->environment->getNrOrgTrab());
        $resul = $this->verificaExclusao($r,$response);
        $resul = $resul->getDataSets()[0]->getRows();
        if(empty($resul)){
            try {
                $this->dataSourceName = "talaocheque_operador";
                $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
                $dataSet = $request->getDataSet();
                $dataSet->setDataSourceName($this->dataSourceName);
                $this->crudService->delete($response, $dataSet);
                $this->entityManager->flush();
                $this->deleteCheque ($r,$response,$request);
                $connection->commit();
            } catch (\Exception $e) {
                $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            }
        }else{
            $response->setError(new DTO\Response\Error("Existe cheque emitido ou cancelado.", 1));
        }
    }

    public function save(DTO\Request $request, DTO\Response $response) {

        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        $dataSet = $request->getDataSet();
        $rows = $dataSet->getRows();
        $row = $rows[0];
        $resul = $this->verificaNumeroCheque($row,$response);
        $resul = $resul->getDataSets()[0]->getRows()[0];
        if($resul['TALAO_INI']==0){
            if($resul['TALAO_FIN']==0){
                if($resul['INTERVALO']==0){
                    try {
            			$this->dataSourceName = "talaocheque";
                        $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
                        $this->crudService->save($dataSourceName, $rows);
                        $this->entityManager->flush();
                        $this->inserirCheque($rows,$response,$request);
                        $connection->commit();
                    } catch (\Exception $e) {
            			$response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
            		}
                }
                else{
                    $response->setError(new DTO\Response\Error("Existe(m) folha(s) de cheque cadastrada(s) para intervalo informado.", 1));

                }
            }
            else{
                $response->setError(new DTO\Response\Error("Nº de cheque Final já cadastrado para a conta informada. ", 1));
            }
        }
        else{
            $response->setError(new DTO\Response\Error("Nº de cheque Inicial já cadastrado para a conta informada. ", 1));
        }
    }

    public function deleteCheque ($rows,$response,$request){
        $this->dataSourceName = "cheque";
        $inicial = intval($rows['NRCHEQINIC']);
        $final =intval($rows['NRCHEQFINA']);
        for($i = $inicial; $i<=$final;$i++){
            $row = new Row();
            $row->set('__is_new',  false);
            $row->set('__createdLocal',true);
            $row->set('changed',true);
            $row->set('CDREDUCONT', $rows['CDREDUCONT']);
            $row->set('NRCHEQUE', str_pad($i, 6, "0", STR_PAD_LEFT));
            $row->set('NRORG',$this->environment->getNrOrgTrab());
            $array[] = $row;
        }

        $dataSet = new DataSet('cheque', $array);
        $request2 = new DTO\Request\dataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
        $request = $this->crudService->buildDataSetIfRow($request2, $this->dataSourceName);
        $dataSet = $request->getDataSet();
        $this->crudService->delete($response, $dataSet);
        $this->entityManager->flush();
    }
    public function inserirCheque($rows,$response,$request){
        $this->dataSourceName = "cheque";
        $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
        $array = array();
        $inicial = intval($rows[0]['NRCHEQINIC']);
        $final =intval($rows[0]['NRCHEQFINA']);
        for($i = $inicial; $i<=$final;$i++){
            $row = new Row();
            $row->set('__is_new',  true);
            $row->set('__createdLocal',true);
            $row->set('changed',true);
            $row->set('IDSTCHEQUE',0);
            $row->set('CDREDUCONT', $rows[0]['CDREDUCONT']);
            $row->set('NRCHEQUE', str_pad($i, 6, "0", STR_PAD_LEFT));
            $row->set('NRORG', '');
            $row->set('IDATIVO', '');
            $row->set('DTULTATU', '');
            $row->set('NRORGULTATU', '');
            $row->set('CDOPERULTATU', '');
            $array[] = $row;
        }
        try {
            $dataSet = new DataSet('cheque', $array);
            $request2 = new DTO\Request\dataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            $saveRows = $this->crudService->getRowsToSave($request2,$dataSourceName);
            $this->crudService->save('cheque', $saveRows);
            $this->entityManager->flush();
        }catch (\Exception $e) {
                $response->setError(new Error("Error in data save. ". $e->getMessage(), 500));
        }

    }
    public function verificaExclusao($row,$response){
        $this->dataSourceName = "talao_verifica_exclusao_cheque";
        $this->parameterBagColumns = array("P_CDREDUCONT", "P_CHEQIN","P_CHEQFIN");
        $condicaoAdicional = $this->retornaCondicaoExclusao($row);
        $resul = $this->realizaConsulta($response,$condicaoAdicional);
        return $resul;
    }

    public function verificaNumeroCheque($row,$response) {
        $this->dataSourceName = "verifica_numero_cheque";
        $this->parameterBagColumns = array("P_CDREDUCONT","P_NRCHEQ_INI","P_NRCHEQ_FIN","P_NRORG");
        $condicaoAdicional = $this->retornaCondicao($row);
        $resul = $this->realizaConsulta($response,$condicaoAdicional);
        return $resul;
    }
    public function retornaCondicao($row) {
        $conditions = array(
            array(
                "value" => $row['CDREDUCONT'],
                "operator" => "=",
                "columnName" => "P_CDREDUCONT"
            ),
            array(
                "value" => $row['NRCHEQINIC'],
                "operator" => "=",
                "columnName" => "P_NRCHEQ_INI"
            ),
            array(
                "value" => $row['NRCHEQFINA'],
                "operator" => "=",
                "columnName" => "P_NRCHEQ_FIN"
            )

        );
        return $conditions;
    }

    public function retornaCondicaoExclusao($row) {
        $conditions = array(
            array(
                "value" => $row['CDREDUCONT'],
                "operator" => "=",
                "columnName" => "P_CDREDUCONT"
            ),
            array(
                "value" => $row['NRCHEQINIC'],
                "operator" => "=",
                "columnName" => "P_CHEQIN"
            ),
            array(
                "value" => $row['NRCHEQFINA'],
                "operator" => "=",
                "columnName" => "P_CHEQFIN"
            )
        );
        return $conditions;
    }
    public function realizaConsulta($response,$conditions) {
        $filterCriteria = new FilterCriteria($this->dataSourceName,$conditions);
        //$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
        $newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $this->crudService->find($response,$newFilterCriteria);
        return $response;
    }
}