<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Financeiro\Entities\Movfluxo;
use Teknisa\Model\Financeiro\Entities\Ajbolpag;
use Teknisa\Model\Financeiro\Entities\Ajctpagar;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Ratctpagar;
use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Teknisa\FIN\Modules\ContaPagar\Factory\AjusteContaFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Services\ContabilizacaoService;

/**
 * Class AjusteContaService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class AjusteContaService {

    protected $entityManager;
    protected $ajusteContaFactory;
    protected $fluxoDeCaixaService;
    protected $contabilizacaoService;

    /**
     * @param EntityManager $entityManager
     * @param AjusteContaFactory $ajusteContaFactory
     * @param FluxoDeCaixaService $fluxoDeCaixaService
     * @param ContabilizacaoService $contabilizacaoService
     */
    function __construct(EntityManager $entityManager, AjusteContaFactory $ajusteContaFactory, FluxoDeCaixaService $fluxoDeCaixaService, ContabilizacaoService $contabilizacaoService)
    {
        $this->entityManager         = $entityManager;
        $this->ajusteContaFactory    = $ajusteContaFactory;
        $this->fluxoDeCaixaService   = $fluxoDeCaixaService;
        $this->contabilizacaoService = $contabilizacaoService;
    }

    /**
     * Deleta Ajuste Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $cdtpctpagar
     * @param $nrlanbolpag
     * @param $cdfilirece
     * @param $nrparcelapag
     * @param $nrseqvenpag
     * @param $valorajuste
     * @param $nrlanctonf
     * @param $idajuste
     * @param $vrsaimfluxo
     * @param $vrajuste
     * @param $valoratual
     * @param $dtajustepag
     * @param $cdempresa
     * @param $cdclassfina
     * @param $idmovfluxo
     * @param $cdajuctpag
     * @param $idstatpag
     * @throws \Exception
     */
    public function deletaAjusteConta(
        $nrorg, $cdfilial, $cdtpctpagar, $nrlanbolpag, $cdfilirece, $nrparcelapag, $nrseqvenpag, $valorajuste,
        $nrlanctonf, $idajuste, $vrsaimfluxo, $vrajuste, $valoratual, $dtajustepag, $cdempresa,
        $cdclassfina, $idmovfluxo, $cdajuctpag, $idstatpag
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Atualiza Fluxo de Caixa
            /** @var Movfluxo $movfluxo */
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $nrorg, $cdempresa, $cdfilirece, $cdclassfina, $dtajustepag, $vrsaimfluxo, $idmovfluxo, 'S'
            );
            $this->entityManager->persist($movfluxo);

            // Busca Conta a Pagar
            /** @var $conta Contapagar */
            $conta = $this->ajusteContaFactory->buscaContapagar($nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag);

            // Calcula Valor Conta a Pagat
            if ($idajuste == 'A')
                 $valor = $conta->getVratuapag() - $valorajuste;
            else $valor = $conta->getVratuapag() + $valorajuste;

            // Atualiza Valor Conta a Pagar
            $conta->setVratuapag($valor);
            $this->entityManager->persist($conta);

            // Busca Boleto a Pagar
            /** @var $boleto Boletpagar*/
            $boleto = $this->ajusteContaFactory->buscaBoletpagar($nrorg, $cdfilial, $nrlanbolpag);

            // Atualiza Valor Boleto Pagar
            $boleto->setVrbolpag($valor);
            $boleto->setVrfinalbpag($valor);
            $this->entityManager->persist($boleto);

            // Atualiza Valor Rateio
            self::atualizaValorRateio(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfilirece, $vrajuste, $valoratual, $idajuste, 'E'
            );
            $this->entityManager->flush();

            // Busca Ajuste Conta Pagar
            /** @var $ajctpagar Ajctpagar */
            $ajctpagar = $this->ajusteContaFactory->buscaAjctpagar(
                $nrorg, $cdfilial, $nrlanctonf, $cdajuctpag, $nrseqvenpag, $nrparcelapag
            );

            // Busca Lançamento Contabil
            /** @var $lanccont Lanccont */
            $lanccont = $this->ajusteContaFactory->buscaLanccont($nrorg, $cdempresa, $ajctpagar->getNrlctbajpag());

            // Verifica Existência Lançamento Contábil
            if ($lanccont)
                // Percorre Lançamentos Contábil
                foreach ($lanccont as $lanc)
                    // Deleta Lançamento Contábil
                    $this->entityManager->remove($lanc);

            // Deleta Ajuste
            $this->entityManager->remove($ajctpagar);
            $this->entityManager->flush();

            // Contabiliza Conta
            $this->contabilizacaoService->contabilizaPagamento($cdfilial, $cdempresa, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $idstatpag, $cdtpctpagar);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Atualiza Valor Rateio Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdfiratctpag
     * @param $vrajuste
     * @param $valorAtual
     * @param $idajuste
     * @param $tipoOperacao
     * @throws \Exception
     */
    public function atualizaValorRateio(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfiratctpag, $vrajuste, $valorAtual, $idajuste, $tipoOperacao
    ){
        try {
            // Busca Rateios da Conta
            /** @var $rateios Ratctpagar */
            $rateios = $this->ajusteContaFactory->buscaRatctpagar(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfiratctpag
            );

            // Verifica Existência Rateios
            if ($rateios) {

                // Variáveis Inicialização
                $valor   = 0;
                $somarat = 0;

                // Percorre Todos Rateios
                /** @var $rateio Ratctpagar */
                foreach ($rateios as $rateio) {
                    
                    // Calcula Valor Rateio
                    $valorPercentual = ($rateio->getVrratccctpag() * 100) / $valorAtual;
                    if ($tipoOperacao == 'I') {
                        if ($idajuste == 'A') {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctpag() + $valor;
                        } else {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctpag() - $valor;
                        }
                    } else {
                        if ($idajuste == 'A') {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctpag() - $valor;
                        } else {
                            $valor = $vrajuste * ($valorPercentual / 100);
                            $valor = $rateio->getVrratccctpag() + $valor;
                        }
                    }
                    $somarat += round($valor, 2);

                    // Atualiza Valor Rateio
                    $rateio->setVrratccctpag($valor);
                    $this->entityManager->persist($rateio);
                }
                
                // Verifica Diferença Valor Rateio
                if ($tipoOperacao != 'I' && $idajuste == 'A' or $tipoOperacao == 'I' && $idajuste != 'A')
                     $valorAjustado = $vrajuste * -1;
                else $valorAjustado = $vrajuste;
                $diferenca = round($valorAtual + $valorAjustado - $somarat, 2);

                // Atualiza Diferença Valor Rateio
                if ($diferenca != 0) {
                    $rateio = $rateios[count($rateios) - 1];
                    $rateio->setVrratccctpag($valor + $diferenca);
                    $this->entityManager->persist($rateio);
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

} 