<?php

namespace Teknisa\FIN\Modules\ContaReceber\Controllers;

use Financeiro\Entities\Boletreceber;
use Financeiro\Logic\ContasReceber\Boleto\OperacoesBoleto;
use Financeiro\Logic\ContasReceber\Boleto\ParcelarBoleto\ExcecaoParcelamento;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Service\CRUD as CrudService;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO\Request\DataSet as Data;
use Teknisa\FIN\Modules\ContaReceber\Services\ParcelamentoBoletoService;

/**
 * Class ParcelamentoBoletoController
 * @package Teknisa\FIN\Modules\ContaReceber\Controllers
 */
class ParcelamentoBoletoController extends TeknisaCrud{
    protected $dataSourceName = 'boletreceber_motivoparc_contcorr';
    protected $parameterBagColumns = array(
        "CDFILIAL",
        "NRLANBOLREC",
        "NRORG"
    );

    /**
     * @param CrudService $crudService
     * @param ParameterBag $parameterBag
     * @param Environment $environment
     * @param ParcelamentoBoletoService $parcelamentoboletoservice
     */
    public function __construct(CrudService $crudService, ParameterBag $parameterBag, Environment $environment, ParcelamentoBoletoService $parcelamentoboletoservice)
    {
        $this->crudService  = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment  = $environment;
        $this->parcelamentoBoletoService = $parcelamentoboletoservice;
    }

    /**
     * Finaliza Parcelamento Títulos
     * @param Data $dataset
     * @param Response $response
     */
    public function finalizaParcelamentoTitulos(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $dataset  = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $nrorg       = null;
            $cdfilial    = null;
            $nrlanbolrec = null;
            $parcelas = array();
            
            // Percorre Parcelas
            foreach($dataset as $data){
                
                // Verifica Parcela Inicial
                if(($data['NRPARCBOLREC'] == '00') || ($data['NRPARCBOLREC'] == '01')) {
                    $nrorg       = $data['NRORG'];
                    $cdfilial    = $data['CDFILIAL'];
                    $nrlanbolrec = $data['NRLANBOLREC'];
                }

                // Monta Array Parcelas
                $parcelas[] = array(
                    'NRORG'            => $data['NRORG']       ,
                    'CDFILIAL'         => $data['CDFILIAL']    ,
                    'NRLANBOLREC'      => $data['NRLANBOLREC'] ,
                    'NRORIGBOLREC'     => $data['NRORIGBOLREC'],
                    'NRPARCBOLREC'     => $data['NRPARCBOLREC'],
                    'NRPARCBOLRECVISU' => $data['NRPARCBOLRECVISU'],
                    'DTVENBOLREC'      => $data['DTVENBOLREC'] ,
                    'VRFINALBREC'      => floatval($data['VRFINALBREC']) ,
                    'CDMOTIPARC'       => $data['CDMOTIPARC']
                );
            }
            
            // Gera Parcelamento Título
            $this->parcelamentoBoletoService->geraParcelamentoTitulo(
                $nrorg, $cdfilial, $nrlanbolrec, $parcelas
            );

            // Registra Dados
            $response->addDataSet(new DataSet('ParcelamentoBoletoService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Finaliza Parcelamento Títulos
     * @param Data $dataset
     * @param Response $response
     */
    public function deletaParcelamentoTitulos(Data $dataset, Response $response)
    {
        try {
            // Dados do Registro
            $dataset  = $dataset->getDataSet()->getRows();

            // Variáveis de Inicialização
            $parcelas = array();

            // Percorre Parcelas
            foreach($dataset as $data){
                // Monta Array Parcelas
                $parcelas[] = array(
                    'NRORG'        => $data['NRORG']       ,
                    'CDFILIAL'     => $data['CDFILIAL']    ,
                    'NRLANBOLREC'  => $data['NRLANBOLREC'] ,
                    'NRORIGBOLREC' => $data['NRORIGBOLREC'],
                    'NRPARCBOLREC' => $data['NRPARCBOLREC'],
                    'DTVENBOLREC'  => $data['DTVENBOLREC'] ,
                    'VRFINALBREC'  => floatval($data['VRFINALBREC']) ,
                    'CDMOTIPARC'   => $data['CDMOTIPARC']
                );
            }

            $this->parcelamentoBoletoService->DeletaParcela($parcelas);

            // Registra Dados
            $response->addDataSet(new DataSet('ParcelamentoBoletoService', ['success' => 'S']));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

} 