<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Teknisa\GER\Controllers\TeknisaCrud;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Financeiro\Entities\Tprecboleto;
use Teknisa\FIN\Modules\ContaReceber\Services\TitulosAReceberService as TitulosAReceberService;

class Rec15000ChequeRecebidoService{
    protected $dataSourceManager;
    protected $environment;
    protected $factory;
    protected $newCode;
 
    public function __construct($dataSourceManager, $environment, $factory, $newCode, TitulosAReceberService $titulosRecService) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->factory           = $factory;
        $this->newCode           = $newCode;
        $this->titulosRecService = $titulosRecService;
    }
    
    public function setCustodia($tipo, $rows) {
        // $tipo = B -> Situar em custódia
        // $tipo = A -> Retirar de custódia
        $erro = false;
        $rowsToSave = array();
        if($tipo === 'A') {$esperado = array('B'); $in = true; $exceptionMensagem = ' O(s) cheque(s) não estão em custódia.';}
        else if($tipo === 'B') {$esperado = array('B', 'C'); $in = false; $exceptionMensagem = ' O(s) cheque(s) já estão em custódia ou já foram compensados.';}
        $valido = $this->verificaStatus($rows, $esperado, $in);
        $rows = $valido['rows'];
        $erro = $erro['rows'];
        if(!isset($rows[0])) throw new \Exception('Operação bloqueada.'.$exceptionMensagem);
        else {
            foreach ($rows as $row) {
                $row = $this->factory->getCheque($this->getParams($row))[0];
                $row['IDSITCHEQREC'] = $tipo;
                $row = $this->adicionaCampos($row, false);
                $row['DTULTATUCHRE'] = $row['DTULTATU'];
                array_push($rowsToSave, $row);
            }
            $dataSet = new DataSet('chequerece', $rowsToSave);
            $resul = $this->dataSourceManager->persist($dataSet);
            $resul['erroStatus'] = array($erro);
            return new DataSet('setCustodia', $resul);
        }
    }
    
    public function gerarApresentacao($apresentacao, $rows) {
        $erro = false;
        $rowsToSave = array();
        $rowsToSaveAp = array();
        $valida = $this->verificaStatus($rows, array('P', 'C'), false);
        $rows = $valida['rows'];
        $erro = $valida['erro'];
        if(!isset($rows[0])) throw new \Exception('Operação bloqueada. Os cheques já foram apresentados/compensados.');
        foreach ($rows as $row) {
            $rowUpdate = $this->factory->getCheque($this->getParams($row))[0];
            $rowUpdate = $this->adicionaCampos($rowUpdate, false);
            $rowUpdate['IDSITCHEQREC'] = 'P';
            $rowUpdate['DTULTATUCHRE'] = $rowUpdate['DTULTATU'];
            $rowAp = array();
            $rowAp = $this->setCamposRowAp($apresentacao, $rowUpdate);
            array_push($rowsToSave, $rowUpdate);
            array_push($rowsToSaveAp, $rowAp);
        }
        $dataSet = new DataSet('chequerece', $rowsToSave);
        $dataSetAp = new DataSet('logchequerec', $rowsToSaveAp);
        $resulCheque = $this->dataSourceManager->persist($dataSet);
        $resulAp = $this->dataSourceManager->persist($dataSetAp);
        $resul = array();
        $resul['chequerece'] = $resulCheque;
        $resul['logchequerec'] = $resulAp;
        $resul['erroStatus'] = array($erro);
        return new DataSet('gerarApresentacao', $resul); 
    }
    
    public function gerarDevolucao($apresentacao, $row) {
        $rowAp = array();
        $valida = $this->verificaStatus([$row], array('A', 'C', 'R', 'D', 'B', 'T'), false);
        $rows = $valida['rows'];
        if(!isset($rows[0])) throw new \Exception('Operação bloqueada. Os cheques não estão apresentados.');
        
        $rowAp['NRCHEQUERECE'] = $row['NRCHEQUERECE'];
        $rowAp['CDBANCHEQREC'] = $row['CDBANCHEQREC'];
        $rowAp['CDAGECHEQREC'] = $row['CDAGECHEQREC'];
        $rowAp['CDCNTCHEQREC'] = $row['CDCNTCHEQREC'];
        
        $ordem = $this->factory->getCdOrdem($rowAp)[0]['NRLOGCHEQREC'] - 1;
        $cheque = $this->factory->getCheque($rowAp)[0];
        $ordem = (string)$ordem;
        $rowAp['NRLOGCHEQREC'] = str_pad($ordem, 2, "0", STR_PAD_LEFT);;
        $rowAp = $this->factory->getApresentacao($rowAp)[0];
        $rowAp['DTDEVCHEQREC'] = $apresentacao['DTDEVCHEQREC'];
        $rowAp['DSOBSCHEQREC'] = $apresentacao['DSOBSCHEQREC'];
        $rowAp = $this->adicionaCampos($rowAp, false);
    
        $rowsToSaveAp = array($rowAp);
        $dataSetAp = new DataSet('logchequerec', $rowsToSaveAp);
        $resulAp = $this->dataSourceManager->persist($dataSetAp);
        
        $cheque['IDSITCHEQREC'] = 'D';
        $cheque = $this->adicionaCampos($cheque, false);
        $cheque['DTULTATUCHRE'] = $cheque['DTULTATU'];
        $rowsToSave = array($cheque);
        $dataSet = new DataSet('chequerece', $rowsToSave);
        $resulCheque = $this->dataSourceManager->persist($dataSet);
        
        $resul = array();
        $resul['chequerece'] = $resulCheque;
        $resul['logchequerec'] = $resulAp;
        return new DataSet('gerarDevolucao', $resul); 
    }
    
    public function setBaixa($compensacao, $cheques, $tipo, $response) {
        $valida  = $this->verificaStatus($cheques, array('R', 'C') , false);
        $cheques = $valida['rows'];
        $erro    = $valida['erro'];
        $baixasRecebidas = array();
        $boletoReceber   = array();
        $boletosMarcados = array();
        $geraBaixa = false;
        $chequesCompletos = array();
        foreach ($cheques as $cheque) {
            if($cheque['VRCHEQUERECE'] !== $cheque['VRUTL']) {
                throw new \Exception('Operação bloqueda. Existem cheques que não foram totalmente utilizados.');
            }
            else {
                $cheque = $this->factory->getCheque($this->getParams($cheque))[0];
                $cheque['IDSITCHEQREC'] = 'C';
                $cheque = $this->adicionaCampos($cheque, false);
                array_push($chequesCompletos, $cheque);
                $filter = array(0 => array("value"=> array($cheque['NRCHEQUERECE'])),
                                1 => array("value"=> array($cheque['CDBANCHEQREC'])),
                                2 => array("value"=> array($cheque['CDAGECHEQREC'])),
                                3 => array("value"=> array($cheque['CDCNTCHEQREC'])));
                // Busca boletos para o cheque
                $boletos = $this->getDadosCompensacao($filter, 1);
                $valorBaixa = 0;
                $geraBaixa = null;
                foreach($boletos as $boleto) {
                    if($boletos !== 'B') {
                        $geraBaixa = true;
                        $boletos['__is_new'] = false;
                        array_push($boletosMarcados, $boleto);
                    }
                }
            }
        }
        $filial  = $this->environment->getCdFilial();
        $nrorg   = $this->environment->getNrorgTrab();
        if($geraBaixa) {
            $rowBaixa = array();
            $rowBaixa['VRBAIXAREC']   =  $valorBaixa;
            $rowBaixa['CDFILIAL']     =  $filial;
            $rowBaixa['CDEMPRESA']    =  $chequesCompletos[0]['CDEMPRESA'];
            $rowBaixa['DTBAIXAREC']   =  $compensacao['DTCOMPENSACAO'];
            $rowBaixa['IDSTATUSBREC'] =  $tipo;
            $rowBaixa['DSBAIXAREC']   =  null;
            $rowBaixa['CDOPERADOR']   =  $this->environment->getCdOperador();
            $rowBaixa['CDREDUCONT']   =  $chequesCompletos[0]['CDREDUCONT'];
            $rowBaixa['CDTPBAIXAREC'] =  $compensacao['CDTPBAIXAREC'];
            $rowBaixa['DTLANCBAIXAR'] =  $this->factory->getDtProcess(array("CDFILIAL"=>$filial))[0]['DTPROCESSA'];
            $rowBaixa = $this->adicionaCampos($rowBaixa, true);
            array_push($baixasRecebidas, $rowBaixa);
        }
        $dataSet = new DataSet('chequerece', $chequesCompletos);
        $resulCheque = $this->dataSourceManager->persist($dataSet);
        $this->titulosRecService->baixaTitulo($nrorg, $filial, $rowBaixa, $boletosMarcados, $response);
        return new DataSet('gerarCompensacao', $resulCheque);
    }
    
    public function getDadosCompensacao($filter, $tipo) {
        $sqlFilter = '(TB.NRCHEQBOLREC IN (';
        $sqlFilter.=$this->getSqlFilterValue($filter[0]);
        $sqlFilter.=')) AND (TB.CDBANCBOLREC IN (';
        $sqlFilter.=$this->getSqlFilterValue($filter[1]);
        $sqlFilter.=')) AND (TB.CDAGENBOLREC IN (';
        $sqlFilter.=$this->getSqlFilterValue($filter[2]);
        $sqlFilter.=')) AND (TB.CDCONTBOLREC IN (';
        $sqlFilter.=$this->getSqlFilterValue($filter[3]);
        $sqlFilter.=')))B ';
        if($tipo === 1) return $this->factory->getBoletosCompensacao($sqlFilter);
        if($tipo === 2) return $this->factory->getChequesCompensacao($sqlFilter);
    }
    
    public function deletarApresentacoes($dataSet) {
        $this->dataSourceManager->delete($dataSet);
    }
    
    public function getConditionsDelete($row) {
        $conditions = array(
            array(
                "columnName" => "NRCHEQUERECE",
                "operator" => "=",
                "value" => $row['NRCHEQUERECE']
            ),  
            array(
                "columnName" => "CDBANCHEQREC",
                "operator" => "=",
                "value" => $row['CDBANCHEQREC']
            ),
            array(
                "columnName" => "CDAGECHEQREC",
                "operator" => "=",
                "value" => $row['CDAGECHEQREC']
            ),
            array(
                "columnName" => "CDCNTCHEQREC",
                "operator" => "=",
                "value" => $row['CDCNTCHEQREC']
            )
        );
        return $conditions;
    }
    // Retorna as rows em que o status do cheque(IDSITCHEQREC) é o esperado
    // $possiveis -> array com status em que o cheque deve ou não estar
    // $in        -> diz se o cheque deve estar entre os estatos possiveis ou não
        // $in = $true  -> Status do cheque deve estar no array de possiveis
        // $in = $false -> Status do cheque não deve estar no array de possiveis
    protected function verificaStatus($rows, $possiveis, $in = true) { 
        $rowsReturn = array();
        $erro = false;
        foreach ($rows as $row) {
            if(in_array($row['IDSITCHEQREC'], $possiveis)) {
                if($in) array_push($rowsReturn, $row);
                else $erro = true;
            }
            else {
                if(!$in) array_push($rowsReturn, $row);
                else $erro = true;
            }
        }
        return array("rows"=>$rowsReturn,"erro"=> $erro);
    }
    
    protected function getSqlFilterValue($filter) {
        if($filter['value'] === 'Controller') $filter['value'] = array("'Controller'");
        $sql = '';
        foreach ($filter['value'] as $value) {
            $value.=',';
            $sql.= $value;
        }
        return substr($sql,0,-1);
    }
    
    protected function setCamposRowAp($apresentacao, $row) {
        $rowAp = array();
        $rowAp['NRCHEQUERECE'] = $row['NRCHEQUERECE'];
        $rowAp['CDBANCHEQREC'] = $row['CDBANCHEQREC'];
        $rowAp['CDAGECHEQREC'] = $row['CDAGECHEQREC'];
        $rowAp['CDCNTCHEQREC'] = $row['CDCNTCHEQREC'];
        $ordem = $this->factory->getCdOrdem($rowAp)[0];
        $cdApresentacao = $this->factory->getCdApresentacao()[0];
        $rowAp = $this->adicionaCampos($row, true);
        $rowAp['DTBORDERCREC'] = $apresentacao['DTAPRCHEQREC']; 
        $rowAp['NRLOGCHEQREC'] = $ordem['NRLOGCHEQREC'];
        $rowAp['NRBORDERCREC'] = $cdApresentacao['NRBORDERCREC'];
        $rowAp['DTAPRCHEQREC'] = $apresentacao['DTAPRCHEQREC'];
        if (isset($apresentacao['DSOBSCHEQREC'])){
            $rowAp['DSOBSCHEQREC'] = $apresentacao['DSOBSCHEQREC'];
        }
        $rowAp['CDOPEBORCREC'] = null;
        $rowAp['DTDEVCHEQREC'] = null;
        return $rowAp;
    }
    
    protected function setCamposBoletoReceber($boleto, $compensacao, $cheque) {
        $boleto['CDFILIBAIREC'] = $cheque['CDFILIAL'];
        $boleto['NRBAIXAREC']   = $compensacao['NRBAIXAREC'];
        $boleto['CDTPBAIXAREC'] = $compensacao['CDTPBAIXAREC'];
        $boleto['CDREDUCONT']   = $cheque['CDREDUCONT'];
        $boleto                 = $this->adicionaCampos($boleto, false);
        return $boleto;
    }
    
    public function getParams($row) {
        $params = array();
        $params['NRCHEQUERECE'] = $row['NRCHEQUERECE'];
        $params['CDBANCHEQREC'] = $row['CDBANCHEQREC'];
        $params['CDAGECHEQREC'] = $row['CDAGECHEQREC'];
        $params['CDCNTCHEQREC'] = $row['CDCNTCHEQREC'];
        return $params;
    }
    
    protected function adicionaCampos($row, $new) {
        $nrorg = $this->environment->getNrorgTrab();
		$operador = $this->environment->getCdOperador();
		$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
		$row['NRORG'] = $nrorg;
		if($new === true) {
		    $row['CDOPERINCLUSAO'] = $operador;
		    $row['DTINCLUSAO']     = $currentTime;
		    $row['NRORGINCLUSAO']  = $nrorg;
		    $row['__is_new']       = true;
		}
		else {
		    $row['__is_new']     = false;
		}
        $row['CDOPERULTATU'] = $operador;
        $row['DTULTATU']     = $currentTime;
		$row['NRORGULTATU']  = $nrorg;
		$row['IDATIVO'] = 'S';
		return $row;
    }
    
    public function deleteBoleto($row) {
        $params = array();
        $params['CDFILIAL'] = $row['CDFILIAL'];
        $params['NRLANBOLREC'] = $row['NRLANBOLREC'];
        $params['NRSEQTPRECBO'] = $row['NRSEQTPRECBO'];
        $this->factory->deleteBoleto($params);
    }
    
    public function saveBoleto($row) {
        $params = array();
        $params['CDFILIAL'] = $row['CDFILIAL'];
        $params['NRLANBOLREC'] = $row['NRLANBOLREC'];
        $params['CDTIPORECE'] = $row['CDTIPORECE'];
        $params['NRSEQTPRECBO'] = $row['NRSEQTPRECBO'];
        $params['VRTPRECBOLR'] = $row['VRTPRECBOLR'];
        $params['NRCHEQBOLREC'] = $row['NRCHEQBOLREC'];
        $params['CDBANCBOLREC'] = $row['CDBANCBOLREC'];
        $params['CDAGENBOLREC'] = $row['CDAGENBOLREC'];
        $params['CDCONTBOLREC'] = $row['CDCONTBOLREC'];
        $this->factory->saveBoleto($params);
    }
}