<?php
namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Teknisa\Libs\Util\DateUtil;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Model\Financeiro\Entities\GeralFcx;
use Teknisa\Model\Financeiro\Entities\GeralBco;
use Teknisa\Model\Geral\Entities\Paramgeral;

class SaveGeralBcoParamgeralFactory {
    protected $connection;
    protected $entityManager;

    public function __construct($entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function getGeralBco(){
        $params = array();
        $geralBco = $this->entityManager->getRepository(GeralBco::class)->findOneBy($params);
        return $geralBco;
    }
    
    public function getGeralFcx(){
        $params = array();
        $geralFcx = $this->entityManager->getRepository(GeralFcx::class)->findOneBy($params);
        return $geralFcx;
    }
    
    public function getParamgeral(){
        $params = array();
        $paramgeral = $this->entityManager->getRepository(Paramgeral::class)->findOneBy($params);
        return $paramgeral;
    }
    
    public function popularGeralBco($geralBco, $rowGeralBco){
        $geralBco = new GeralBco();
        $geralBco->setCdgeralbco($rowGeralBco['CDGERALBCO']);
        $geralBco->setNrorg($rowGeralBco['NRORG']);
        return $geralBco;
    }

    /**
     * Insere Parâmetros Gerais
     * @param $rowParamgeral
     */
    public function insereParamgeral($rowParamgeral){
        $paramgeral = new Paramgeral();
        
        $paramgeral->setIdutlprocbco($rowParamgeral['IDUTLPROCBCO']);
        $paramgeral->setCdgroperprob($rowParamgeral['CDGROPERPROB']);
        $paramgeral->setCdpargeral($rowParamgeral['CDPARGERAL']);
        $paramgeral->setNrorg($rowParamgeral['NRORG']);
        
        return $paramgeral;
    }
    
    /**
     * Insere Parâmetros Controle Bancário
     * @param $rowParamgeral
     */
    public function insereGeralBco($rowGeralBco){
        $geralBco = new GeralBco();
        
        $geralBco->setCdgeralbco($rowGeralBco['CDGERALBCO']);
        $geralBco->setIdctbtrans($rowGeralBco['IDCTBTRANS']);
        $geralBco->setIdtpimprutl($rowGeralBco['IDTPIMPRUTL']);
        $geralBco->setIdtpportimpr($rowGeralBco['IDTPPORTIMPR']);
        $geralBco->setQtdiaantlanc($rowGeralBco['QTDIAANTLANC']);
        $geralBco->setQtdiaapolanc($rowGeralBco['QTDIAAPOLANC']);
        $geralBco->setQtdiaanttran($rowGeralBco['QTDIAANTTRAN']);
        $geralBco->setQtdiaapotran($rowGeralBco['QTDIAAPOTRAN']);
        $geralBco->setIdpertransccor($rowGeralBco['IDPERTRANSCCOR']);
        $geralBco->setQtdiaanttran($rowGeralBco['QTDIAANTTRAN']);
        $geralBco->setNrorg($rowGeralBco['NRORG']);
        
        return $geralBco;
    }
    
    /**
     * Insere Parâmetros Fluxo de Caixa
     * @param $rowParamgeral
     */
    public function insereGeralFcx($rowGeralFcx){
        $geralFcx = new GeralFcx();
        
        $geralFcx->setCdgeralfcx($rowGeralFcx['CDGERALFCX']);
        $geralFcx->setCdclassfina($rowGeralFcx['CDCLASSFINA']);
        $geralFcx->setNrorg($rowGeralFcx['NRORG']);
        
        return $geralFcx;
    }
    
    /**
     * Atualiza Parâmetros Gerais
     * @param $paramgeral
     * @param $rowParamgeral
     */
    public function atualizaParamgeral($paramgeral, $rowParamgeral){
        $paramgeral->setIdutlprocbco($rowParamgeral['IDUTLPROCBCO']);
        $paramgeral->setCdgroperprob($rowParamgeral['CDGROPERPROB']);
        
        return $paramgeral;
    }
    
    /**
     * Atualiza Parâmetros Controle Bancário
     * @param $paramgeral
     * @param $rowParamgeral
     */
    public function atualizaGeralBco($geralBco, $rowGeralBco){
        $geralBco->setIdctbtrans($rowGeralBco['IDCTBTRANS']);
        $geralBco->setIdtpimprutl($rowGeralBco['IDTPIMPRUTL']);
        $geralBco->setIdtpportimpr($rowGeralBco['IDTPPORTIMPR']);
        $geralBco->setQtdiaantlanc($rowGeralBco['QTDIAANTLANC']);
        $geralBco->setQtdiaapolanc($rowGeralBco['QTDIAAPOLANC']);
        $geralBco->setQtdiaanttran($rowGeralBco['QTDIAANTTRAN']);
        $geralBco->setQtdiaapotran($rowGeralBco['QTDIAAPOTRAN']);
        $geralBco->setIdpertransccor($rowGeralBco['IDPERTRANSCCOR']);
        $geralBco->setQtdiaanttran($rowGeralBco['QTDIAANTTRAN']);
        
        return $geralBco;
    }
    
    /**
     * Atualiza Parâmetros Fluxo de Caixa
     * @param $paramgeral
     * @param $rowParamgeral
     */
    public function atualizaGeralFcx($geralFcx, $rowGeralFcx){
        $geralFcx->setCdclassfina($rowGeralFcx['CDCLASSFINA']);
        
        return $geralFcx;
    }
}